# This file is part of 'PyAsync'
# Copyright 2000 Constantinos A. Kotsokalis <ckotso@cs.ntua.gr>
#
# PyAsync is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PyAsync is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyAsync; see the file COPYING. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import socket, sys, re
try:
	import threading
except ImportError:
	sys.stderr.write("""
You will need a thread-safe Python installation in
order to use CTC. You do not seem to have one though!
Go to http://www.python.org/ -- or the closest mirror
site --, download the latest Python version, unpack it,
and type ./configure --with-threads. Then type make and
make install.

""")
	sys.exit(1)

import dnsthread

BLOCKING = 1

class dnscache:
	
	def __init__(self, errobj):
		self.errobj = errobj
		self.names = self.load_cache()
		self.re_ip = re.compile("^[12][0-9]{0,2}(\.(0|([12][0-9]{0,2}))){3,3}$")
		self.lv = threading.Lock()


	def name_add_change(self, hostname, ip):
		self.lv.acquire(BLOCKING)
		self.names[hostname] = ip
		self.lv.release()
		return

	
	def name_del(self, hostname):
		self.lv.acquire(BLOCKING)
		del(self.names[hostname])
		self.lv.release()
		return
	

	def name_check(self, hostname):
		if self.re_ip.match(hostname):
			return hostname
		elif self.names.has_key(hostname):
			return self.names[hostname]
		else:
			try:
				ipaddr = socket.gethostbyname(hostname)
			except socket.error, msg:
				self.errobj.eprint("gethostbyname: ", msg)
				return hostname
			self.lv.acquire(BLOCKING)
			self.names[hostname] = ipaddr
			self.lv.release()
			return ipaddr


	def load_cache(self):
		names = {'localhost':'127.0.0.1'}
		return names


	def dns_update(self):
		dns_thread = threading.Thread(None, dnsthread.dns_thread, None, (self.lv, self.names), {})
		dns_thread.start()
		return

